﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Runtime.InteropServices;

namespace CUEImporter
{
    public sealed class AudioLengthGetter : IDisposable
    {
        private static AudioLengthGetter instance = null;
        public bool isOK = false;
        public static AudioLengthGetter getInstance()
        {
            if (instance == null) {
                instance = new AudioLengthGetter();
            }
            return instance;
        }
        private AudioLengthGetter()
        {
            bool Ok;
            if (BASS.init() == false) {
                isOK = false;
                return;
            }
            string[] pinnames = { "bassflac.dll", "bass_tta.dll", "bass_wv.dll", "bass_ape.dll", "bass_wma.dll", "bass_aac.dll", "bass_alac.dll", "bass_tak.dll", "plugin\\bassflac.dll", "plugin\\bass_tta.dll", "plugin\\bass_wv.dll", "plugin\\bass_ape.dll", "plugin\\bass_wma.dll", "plugin\\bass_aac.dll", "plugin\\bass_alac.dll", "plugin\\bass_tak.dll", };
            foreach (string pinname in pinnames) {
                Ok = BASS.BASS_PluginLoad(pinname, 0);
                if (Ok == true) {
                    Form1.pTrace(pinname + " を読み込みました");
                }
            }
            isOK = true;
        }

        public float getLength(string filename)
        {
            IntPtr stream = (IntPtr)0;
            UInt64 pos = 0;
            float sec = 0;
            try {
                for (int i = 0; i < 10; i++) { // 100msスリープしながら10回リトライ
                    try {
                        stream = BASS.BASS_StreamCreateFile(false, filename, 0, 0, 0);
                        pos = BASS.BASS_ChannelGetLength(stream);
                        sec = BASS.BASS_ChannelBytes2Seconds(stream, pos);
                        if (sec > 0) break;
                    } catch (Exception) {
                        System.Threading.Thread.Sleep(100);
                    }
                }
            } catch (Exception) {
            } finally {
                BASS.BASS_StreamFree(stream);
            }
            return sec;
        }
        public void Dispose()
        {
            BASS.Dispose();
        }
    }
}
namespace CUEImporter{
    public class BASS
    {
        public static uint BASS_DEVICE_NOSPEAKER = 4096;
        private static uint BASS_UNICODE = 0x80000000;
        private static List<IntPtr> plugins = new List<IntPtr>();
        public static bool init()
        {
            try {
                BASS.BASS_Init(-1, 44100, BASS.BASS_DEVICE_NOSPEAKER, (IntPtr)0, (IntPtr)0);
                return true;
            } catch {
                return false;
            }
        }

        public static bool BASS_PluginLoad(string filename, uint flags)
        {
            IntPtr pinPtr = (IntPtr)0;
            try {
                pinPtr = _BASS_PluginLoad(filename, BASS_UNICODE | flags);
                if (pinPtr != (IntPtr)0) {
                    plugins.Add(pinPtr);
                    return true;
                } else {
                    return false;
                }
            } catch (Exception) {
                return false;
            }
        }

        public static IntPtr BASS_StreamCreateFile(Boolean ismemory, string filename, uint offset, uint length, uint flags)
        {
            try {
                return _BASS_StreamCreateFile(ismemory, filename, offset, length, BASS_UNICODE | flags);
            } catch (Exception) {
                return (IntPtr)0;
            }
        }

        public static bool BASS_StreamFree(IntPtr stream)
        {
            bool ret = false;
            try {
                ret = _BASS_StreamFree(stream);
            } catch (Exception) {
                ret = false;
            }
            return ret;
        }

        public static void Dispose()
        {
            foreach (IntPtr plugin in plugins) {
                try {
                    _BASS_PluginFree(plugin);
                } catch { }
            }
            plugins.Clear();
            try {
                _BASS_Free();
            } catch {
            }
        }
        [DllImport("bass.dll", EntryPoint = "BASS_Init")]
        private static extern Boolean BASS_Init(int device, uint freq, uint flags, IntPtr hwnd, IntPtr guid);

        [DllImport("bass.dll", EntryPoint = "BASS_PluginLoad", CharSet = CharSet.Unicode)]
        private static extern IntPtr _BASS_PluginLoad(string filename, uint flags);

        [DllImport("bass.dll", EntryPoint = "BASS_PluginFree", CharSet = CharSet.Unicode)]
        private static extern bool _BASS_PluginFree(IntPtr plugin);

        [DllImport("bass.dll", EntryPoint = "BASS_StreamCreateFile", CharSet = CharSet.Unicode)]
        private static extern IntPtr _BASS_StreamCreateFile(Boolean ismemory, string filename, uint offset, uint length, uint flags);

        [DllImport("bass.dll", EntryPoint = "BASS_StreamFree")]
        private static extern bool _BASS_StreamFree(IntPtr stream);

        [DllImport("bass.dll", EntryPoint = "BASS_ChannelGetLength")]
        public static extern UInt64 BASS_ChannelGetLength(IntPtr handle);

        [DllImport("bass.dll", EntryPoint = "BASS_ChannelBytes2Seconds")]
        public static extern float BASS_ChannelBytes2Seconds(IntPtr handle, UInt64 pos);

        [DllImport("bass.dll", EntryPoint = "BASS_ErrorGetCode")]
        public static extern int BASS_ErrorGetCode();

        [DllImport("bass.dll", EntryPoint = "BASS_Free")]
        private static extern bool _BASS_Free();
    }
}
