﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace CUEImporter
{
    public delegate void dTrace(string str);
    public partial class Form1 : Form
    {
        public static dTrace pTrace;
        private volatile Queue<CD> SQLQue;
        private volatile Queue<string> fileLoadQue;
        SQLite3DB libraryDB;
        Thread importThread = null;
        private static int loadThreadCount = 4;
        Thread[] loadThreads;

        private void ProgressStep(ProgressBar bar)
        {
            bar.PerformStep();
        }
        private void ProgressFin(ProgressBar bar)
        {
            bar.Value = 0;
            bar.Maximum = 0;
        }
        private void ProgressAddMax(ProgressBar bar, int num)
        {
            bar.Maximum += num;
        }
        public delegate void DPVoid(ProgressBar bar);
        public delegate void DPInt(ProgressBar bar,int i);
        public void trace(string str)
        {
            if (myconsole.InvokeRequired) {
                dTrace d = new dTrace(trace);
                this.Invoke(d, new object[] { str });
            } else {
                myconsole.AppendText(str + "\r\n");
            }
        }
        public Form1()
        {
            InitializeComponent();
        }
        private void setTitle(string str)
        {
            if (this.InvokeRequired) {
                dTrace d = new dTrace(setTitle);
                this.Invoke(d, new object[] { str });
            } else {
                this.Text="CUEシート インポータ"+str;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            pTrace = new dTrace(trace);
            SQLQue = new Queue<CD>();
            fileLoadQue = new Queue<string>();
            loadThreads = new Thread[loadThreadCount];
            Form1.pTrace(((AudioLengthGetter.getInstance().isOK) ? "Bass.dll ロード成功" : "Bass.dll ロードできませんでした"));
            for (int i = 0; i < loadThreadCount; i++) {
                loadThreads[i] = new Thread(runLoad);
                loadThreads[i].Start();
            }
        }

        private void runLoad()
        {
            bool isWorking = false;
            bool isOK;
            int i=0;
            string filename;
            while (true) {
                Thread.Sleep(10);
                try {
                    if (fileLoadQue.Count > 0) {
                        isWorking = true;
                        setTitle(" 処理中... 残り" + (progressBarLoad.Maximum - progressBarLoad.Value) + "ファイル");
                        lock (fileLoadQue) {
                            filename = fileLoadQue.Dequeue();
                        }
                        isOK = treatCUE(filename);
                        this.Invoke(new DPVoid(ProgressStep), new object[] { progressBarLoad });
                        setTitle(" 処理中... 残り" + (progressBarLoad.Maximum - progressBarLoad.Value) + "ファイル");
                        if (isOK) { i++; }
                    } else {
                        if (isWorking) {
                            if (progressBarLoad.Maximum == progressBarLoad.Value) {
                                setTitle("");
                                this.Invoke(new DPVoid(ProgressFin), new object[] { progressBarLoad });
                            }
                            i = 0;
                        }
                        Thread.Sleep(100);
                    }
                }catch(ThreadAbortException){
                    return;
                } catch (Exception ex) {
                    trace("エラー" + ex.Message);
                }
            }
        }
        private void runImport() // import thread
        {
            int inserted = 0;
            int updated = 0;
            int c = 0;
            try {
                libraryDB.Exec("BEGIN;");
                trace("library.dbへのインポートを開始しました");
            } catch {
                trace("library.dbへのインポートを開始できませんでした");
                return;
            }
            c = SQLQue.Count;
            CD currentCD;
            string currentSQL;
//            progressBarInsert.Maximum = SQLQue.Count;
            object[] _progressBarInsert = { progressBarInsert };
            this.Invoke(new DPInt(ProgressAddMax), new object[] { progressBarInsert, SQLQue.Count });
            while (SQLQue.Count > 0) {
                lock(SQLQue){
                    currentCD = SQLQue.Dequeue();
                }
                currentSQL = currentCD.ToINSERTString();
                this.Invoke(new DPVoid(ProgressStep), _progressBarInsert);
                try {
                    libraryDB.Exec(currentSQL);
                    inserted++;
                } catch (SQLite3DB.SQLite3Exception) {
                    try {
                        libraryDB.Exec(currentCD.ToUPDATEString());
                        updated++;
                    } catch (SQLite3DB.SQLite3Exception) {
                    }
                }
            }
            try {
                libraryDB.Exec("COMMIT;");
                this.Invoke(new DPVoid(ProgressFin), _progressBarInsert);
                trace(String.Format("library.dbへのインポートが完了しました {0}/{1}ファイル成功 (INSERT={2}, UPDATE={3}) ",c,(inserted+updated),inserted,updated));
            } catch {
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int i;
            OpenFileDialog fileDLG = new OpenFileDialog();
            fileDLG.Filter = "CUEシート|*.cue";
            fileDLG.DefaultExt = "CUE";
            fileDLG.AddExtension = true;
            fileDLG.Multiselect = true;
            if(fileDLG.ShowDialog() == DialogResult.OK){
                for (i = 0; i < fileDLG.FileNames.Length; i++) {
                    lock (fileLoadQue) {
                        fileLoadQue.Enqueue(fileDLG.FileNames[i]);
                        this.Invoke(new DPInt(ProgressAddMax), new object[]{progressBarLoad,1});
                    }
                }
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            for (int i = 0; i < loadThreadCount; i++) {
                loadThreads[i].Abort();
            }
            if(libraryDB != null)libraryDB.Close();
        }

        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop)) {
                foreach (string filename in (string[])e.Data.GetData(DataFormats.FileDrop)) {
                    lock (fileLoadQue) {
                        fileLoadQue.Enqueue(filename);
                        this.Invoke(new DPInt(ProgressAddMax), new object[]{ progressBarLoad,1 });
                    }
                }
            }

        }
        private bool treatCUE(string filename)
        {
            StringBuilder str = new StringBuilder(filename+"\r\n");
//            str.AppendLine(filename);
            CD mycd = CUEparse.openFile(filename);
            if (mycd != null) {
                if (!mycd.ToINSERTString().Equals("")) {
                    lock (SQLQue) { SQLQue.Enqueue(mycd); }
                    int i = 0;
                    str.AppendLine(" " + mycd.album + " / " + mycd.artist);
                    foreach (CD.Track track in mycd.tracks) {
                        if ((track.filename != null) && (!track.filename.Equals(""))) {
                            str.AppendLine("  " + (++i) + ". " + track.title + " - " + track.artist);
                        } else { break; }
                    }
                } else {
                    str.AppendLine("  CUEシートが無効です(おそらく実体のファイルが存在しません)");
                    trace(str.ToString());
                    return false;
                }
            }
            lock (myconsole) {
                trace(str.ToString());
            }
            return true;
        }

        private void Form1_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if ((importThread != null)&&(importThread.ThreadState != ThreadState.Stopped)) {
                return;
            }
            if (libraryDB == null) {
                OpenFileDialog openDLG = new OpenFileDialog();
                openDLG.Filter = "library.dbファイル|library.db";
                openDLG.DefaultExt = "DB";
                if (openDLG.ShowDialog() == DialogResult.OK) {
                    try {
                        libraryDB = new SQLite3DB(openDLG.FileName);
                    } catch (SQLite3DB.SQLite3Exception ex) {
                        trace(ex.Message);
                    }
                }
            }
            importThread = new Thread(runImport);
            importThread.Start();
        }
    }
}


