﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace CUEImporter
{
    public class SQLite3DB : IDisposable
    {
        private IntPtr dbPtr = (IntPtr)0;
        private const int SQLITE3_OK = 0;

        // Exception
        public class SQLite3Exception : System.ApplicationException
        {
            public SQLite3Exception(string message)
                : base(message)
            {
            }
        }

        // Constructor
        public SQLite3DB(string filename) {
            int ret;
            try {
                ret = sqlite3_open(UTF16toUTF8(filename), out dbPtr); //UTF16toUTF8( 
            } catch (Exception) {
                throw (new SQLite3Exception("db file open error."));
            }
            if (ret != SQLITE3_OK) {
                throw (new SQLite3Exception("db file open error."));
            }
        }

        public void Dispose(){
            if (dbPtr != (IntPtr)0) {
                sqlite3_close(dbPtr);
                dbPtr = (IntPtr)0;
            }
        }
        public void Close()
        {
            Dispose();
        }
        ~SQLite3DB()
        {
            Dispose();
        }

        public void Exec(string sql)
        {
            StringBuilder err;
            int ret;
            ret = sqlite3_exec(dbPtr, UTF16toUTF8(sql), (IntPtr)0, (IntPtr)0, out err);
            if (ret != SQLITE3_OK) {
                throw (new SQLite3Exception("exec error."));
            }
        }
        // DLL Import
        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_open", CharSet = CharSet.Ansi)]
        private static extern int sqlite3_open(byte[] filename,out IntPtr db);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_close")]
        private static extern int sqlite3_close(IntPtr db);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_exec",CharSet=CharSet.Ansi)]
        private static extern int sqlite3_exec(IntPtr db,byte[] sql,IntPtr p1,IntPtr p2,out StringBuilder err);

        private static byte[] UTF16toUTF8(string utf16str)
        {
            byte[] unicodeBytes = Encoding.Unicode.GetBytes(utf16str);
            byte[] utf8Bytes = Encoding.Convert(Encoding.Unicode, Encoding.UTF8, unicodeBytes);
            return utf8Bytes;
        }
    }
}
